@@ -8,16 +8,19 @@ from benchopt.cli.completion import complete_solvers
 from benchopt.cli.completion import complete_datasets
 from benchopt.cli.completion import complete_benchmarks
 from benchopt.cli.completion import complete_conda_envs
+from benchopt.cli.completion import complete_config_files
+from benchopt.utils.misc import get_benchopt_requirement
 from benchopt.utils.conda_env_cmd import list_conda_envs
 from benchopt.utils.conda_env_cmd import create_conda_env
 from benchopt.utils.shell_cmd import _run_shell_in_conda_env
 from benchopt.utils.conda_env_cmd import get_benchopt_version_in_env
 from benchopt.utils.profiling import print_stats
+from benchopt.utils.slurm_executor import set_slurm_launch
 
 
 main = click.Group(
-    name='Principal Commands',
-    help="Principal commands that are used in ``benchopt``."
+    name='Main commands',
+    help="Main commands that are used in ``benchopt``."
 )
 
 
@@ -49,11 +52,13 @@ def _get_run_args(cli_kwargs, config_file_kwargs):
         "n_repetitions",
         "timeout",
         "n_jobs",
+        "slurm",
         "plot",
         "html",
         "pdb",
         "profile",
         "env_name",
+        "output",
         "objective_filter",
         "old_objective_filter",
     ]
@@ -66,15 +71,16 @@ def _get_run_args(cli_kwargs, config_file_kwargs):
     "in a benchmark-dedicated conda environment or in your own "
     "conda environment, see the command `benchopt install`."
 )
-@click.argument('benchmark', type=click.Path(exists=True),
+@click.argument('benchmark', default=Path.cwd(), type=click.Path(exists=True),
                 shell_complete=complete_benchmarks)
 @click.option('--objective-filter',
               metavar='<objective_filter>', multiple=True, type=str,
               help="Deprecated alias for `--objective`.")
 @click.option('--objective', '-o',
               metavar='<objective_filter>', multiple=True, type=str,
-              help="Filter the objective based on its parametrized name. This "
-              "can be used to only include one set of parameters.")
+              help="Select the objective based on its parameters, with the "
+              "syntax `objective[parameter=value]`. This can be used to only "
+              "include one set of parameters. ")
 @click.option('--old-objective-filter', '-p',
               multiple=True, type=str,
               help="Deprecated alias for --objective_filter/-o.")
@@ -82,8 +88,10 @@ def _get_run_args(cli_kwargs, config_file_kwargs):
               metavar="<solver_name>", multiple=True, type=str,
               help="Include <solver_name> in the benchmark. By default, all "
               "solvers are included. When `-s` is used, only listed solvers"
-              " are included. To include multiple solvers, "
-              "use multiple `-s` options.", shell_complete=complete_solvers)
+              " are included. Note that <solver_name> can include parameters,"
+              " with the syntax `solver[parameter=value]`. "
+              "To include multiple solvers, use multiple `-s` options.",
+              shell_complete=complete_solvers)
 @click.option('--force-solver', '-f',
               metavar="<solver_name>", multiple=True, type=str,
               help="Force the re-run for <solver_name>. This "
@@ -94,25 +102,32 @@ def _get_run_args(cli_kwargs, config_file_kwargs):
               metavar="<dataset_name>", multiple=True, type=str,
               help="Run the benchmark on <dataset_name>. By default, all "
               "datasets are included. When `-d` is used, only listed datasets"
-              " are included. Note that <dataset_name> can be a regexp. "
+              " are included. Note that <dataset_name> can include parameters,"
+              " with the syntax `dataset[parameter=value]`. "
               "To include multiple datasets, use multiple `-d` options.",
               shell_complete=complete_datasets)
 @click.option('--n-jobs', '-j',
               metavar="<int>", default=1, show_default=True, type=int,
               help='Maximal number of workers to run the benchmark in '
               'parallel.')
+@click.option('--slurm',
+              metavar="<slurm_config.yml>", default=None,
+              help='Run the computation using submitit on a SLURM cluster. '
+              'The YAML file provided to this argument is used to setup the '
+              'SLURM job. See :ref:`slurm_run` for a detailed description.')
 @click.option('--max-runs', '-n',
               metavar="<int>", default=100, show_default=True, type=int,
               help='Maximal number of runs for each solver. This corresponds '
               'to the number of points in the time/accuracy curve.')
 @click.option('--n-repetitions', '-r',
-              metavar='<int>', default=5, show_default=True, type=int,
+              metavar='<int>', default=1, show_default=True, type=int,
               help='Number of repetitions that are averaged to estimate the '
               'runtime.')
 @click.option('--timeout',
               metavar="<int>", default=100, show_default=True, type=int,
               help='Timeout a solver when run for more than <timeout> seconds')
 @click.option('--config', 'config_file', default=None,
+              shell_complete=complete_config_files,
               help="YAML configuration file containing benchmark options.")
 @click.option('--plot/--no-plot', default=True,
               help="Whether or not to plot the results. Default is True.")
@@ -143,6 +158,16 @@ def _get_run_args(cli_kwargs, config_file_kwargs):
               help="Run the benchmark in the conda environment "
               "named <env_name>. To install the required solvers and "
               "datasets, see the command `benchopt install`.")
+@click.option("--output", default="None", type=str,
+              help="Filename for the result output. "
+              "If given, the results will "
+              "be stored at <BENCHMARK>/outputs/<filename>.parquet, "
+              "if another result file has the same name, a number is happened "
+              "to distinguish them "
+              "(ex: <BENCHMARK>/outputs/<filename>_1.parquet)."
+              " If not provided, the output will be saved as "
+              "<BENCHMARK>/outputs/benchopt_run_<timestamp>.parquet."
+              )
 def run(config_file=None, **kwargs):
     if config_file is not None:
         with open(config_file, "r") as f:
@@ -153,9 +178,9 @@ def run(config_file=None, **kwargs):
     # XXX - Remove old and deprecated objective filters in version 1.3
     (
         benchmark, solver_names, forced_solvers, dataset_names,
-        objective_filters, max_runs, n_repetitions, timeout, n_jobs,
-        plot, html, pdb, do_profile, env_name, deprecated_objective_filters,
-        old_objective_filters
+        objective_filters, max_runs, n_repetitions, timeout, n_jobs, slurm,
+        plot, html, pdb, do_profile, env_name, output,
+        deprecated_objective_filters, old_objective_filters
     ) = _get_run_args(kwargs, config)
 
     if len(old_objective_filters):
@@ -172,41 +197,78 @@ def run(config_file=None, **kwargs):
         )
         objective_filters = deprecated_objective_filters
 
-    from benchopt.runner import run_benchmark
-
-    if do_profile:
-        from benchopt.utils.profiling import use_profile
-        use_profile()  # needs to be called before validate_solver_patterns
-
-    # Check that the dataset/solver patterns match actual dataset
+    # Create the Benchmark object
     benchmark = Benchmark(benchmark)
-    benchmark.validate_dataset_patterns(dataset_names)
-    benchmark.validate_objective_filters(objective_filters)
-    # pyyaml returns tuples: solver_names can be tuple and forced_solvers list
-    benchmark.validate_solver_patterns(
-        list(solver_names) + list(forced_solvers)
-    )
+
+    if benchmark.min_version is not None:
+        from packaging.version import parse
+        from benchopt import __version__
+
+        # Avoid dev versions
+        normalized_version = parse(parse(__version__).base_version)
+        if normalized_version < parse(benchmark.min_version):
+            raise RuntimeError(
+                f"benchopt version {__version__} is too old to run this  "
+                f"benchmark, version {benchmark.min_version} is required. "
+                "Please update benchopt with `pip install -U benchopt` "
+                "for this benchmark."
+            )
 
     # If env_name is False, the flag `--local` has been used (default) so
     # run in the current environment.
     if env_name == 'False':
+
+        print("Benchopt is running")
+        if slurm is not None:
+            print("Running on SLURM")
+            set_slurm_launch()
+
+        from benchopt.runner import run_benchmark
+
+        if do_profile:
+            from benchopt.utils.profiling import use_profile
+            use_profile()  # needs to be called before validate_solver_patterns
+
+        # Check that the objective is installed or raise an error
+        objective = benchmark.get_benchmark_objective()
+        objective.is_installed(raise_on_not_installed=True)
+
+        # Check that the dataset/solver patterns match actual dataset
+        benchmark.validate_dataset_patterns(dataset_names)
+        benchmark.validate_objective_filters(objective_filters)
+        # pyyaml returns tuples: make sure everything is a list
+        benchmark.validate_solver_patterns(
+            list(solver_names) + list(forced_solvers)
+        )
+
         run_benchmark(
             benchmark, solver_names, forced_solvers,
             dataset_names=dataset_names,
             objective_filters=objective_filters,
             max_runs=max_runs, n_repetitions=n_repetitions,
-            timeout=timeout, n_jobs=n_jobs,
-            plot_result=plot, html=html, pdb=pdb
+            timeout=timeout, n_jobs=n_jobs, slurm=slurm,
+            plot_result=plot, html=html, pdb=pdb,
+            output=output
         )
 
         print_stats()  # print profiling stats (does nothing if not profiling)
 
         return
 
-    _, all_conda_envs = list_conda_envs()
+    default_conda_env, all_conda_envs = list_conda_envs()
+
     # If env_name is True, the flag `--env` has been used. Create a conda env
     # specific to the benchmark (if not existing).
     # Else, use the <env_name> value.
+
+    # check if any current conda environment
+    if default_conda_env is None:
+        raise RuntimeError(
+            "No conda environment is activated. "
+            "You should be in a conda environment to use "
+            "'benchopt run' with options '-e/--env' or '--env-name'."
+        )
+
     if env_name == 'True':
         env_name = f"benchopt_{benchmark.name}"
         install_cmd = f"`benchopt install -e {benchmark.benchmark_dir}`"
@@ -229,15 +291,26 @@ def run(config_file=None, **kwargs):
             "benchmark and install the dependencies."
         )
 
+    print(f"Launching benchopt in env {env_name}")
+
     # check if environment was set up with benchopt
-    if get_benchopt_version_in_env(env_name) is None:
+    benchopt_version, is_editable = get_benchopt_version_in_env(env_name)
+    if benchopt_version is None:
         raise RuntimeError(
             f"benchopt is not installed in env '{env_name}', "
             "see the command `benchopt install` to setup the environment."
         )
+    # check against running version
+    from benchopt import __version__ as benchopt_version_running
+    _, is_editable_running = get_benchopt_requirement()
+    if (benchopt_version_running != benchopt_version and not
+            (is_editable_running and is_editable)):
+        warnings.warn(
+            f"Benchopt running version ({benchopt_version_running}) "
+            f"and version in env {env_name} ({benchopt_version}) differ")
 
     # run the command in the conda env
-    solvers_option = ' '.join(['-s ' + s for s in solver_names])
+    solvers_option = ' '.join([f"-s '{s}'" for s in solver_names])
     forced_solvers_option = ' '.join([f"-f '{s}'" for s in forced_solvers])
     datasets_option = ' '.join([f"-d '{d}'" for d in dataset_names])
     objective_option = ' '.join([f"-o '{o}'" for o in objective_filters])
@@ -245,12 +318,13 @@ def run(config_file=None, **kwargs):
         rf"benchopt run --local {benchmark.benchmark_dir} "
         rf"--n-repetitions {n_repetitions} "
         rf"--max-runs {max_runs} --timeout {timeout} "
-        rf"--n-jobs {n_jobs} "
+        rf"--n-jobs {n_jobs} {'--slurm' if slurm else ''} "
         rf"{solvers_option} {forced_solvers_option} "
         rf"{datasets_option} {objective_option} "
         rf"{'--plot' if plot else '--no-plot'} "
         rf"{'--html' if html else '--no-html'} "
         rf"{'--pdb' if pdb else ''} "
+        rf"--output {output}"
         .replace('\\', '\\\\')
     )
     raise SystemExit(_run_shell_in_conda_env(
@@ -261,7 +335,7 @@ def run(config_file=None, **kwargs):
 @main.command(
     help="Install the requirements (solvers/datasets) for a benchmark."
 )
-@click.argument('benchmark', type=click.Path(exists=True),
+@click.argument('benchmark', default=Path.cwd(), type=click.Path(exists=True),
                 shell_complete=complete_benchmarks)
 @click.option('--force', '-f',
               is_flag=True,
@@ -286,10 +360,15 @@ def run(config_file=None, **kwargs):
               "datasets are included, except when -s flag is used. "
               "If -s flag is used, then no dataset is included. "
               "When `-d` is used, only listed datasets "
-              "are included. Note that <dataset_name> can be a regexp. "
+              "are included. Note that <dataset_name> can include parameters "
+              "with the syntax `dataset[parameter=value]`. "
               "To include multiple datasets, use multiple `-d` options."
               "To include all datasets, use -d 'all' option.",
               shell_complete=complete_datasets)
+@click.option('--config', 'config_file', default=None,
+              shell_complete=complete_config_files,
+              help="YAML configuration file containing benchmark options, "
+              "whose solvers and datasets will be installed.")
 @click.option('--env', '-e', 'env_name',
               flag_value='True', type=str, default='False',
               help="Install all requirements in a dedicated "
@@ -313,38 +392,55 @@ def run(config_file=None, **kwargs):
               help="If this flag is set, no confirmation will be asked "
               "to the user to install requirements in the current environment."
               " Useless with options `-e/--env` or `--env-name`.")
-def install(benchmark, minimal, solver_names, dataset_names, force=False,
-            recreate=False, env_name='False', confirm=False, quiet=False):
+def install(
+        benchmark, minimal, solver_names, dataset_names, config_file=None,
+        force=False, recreate=False, env_name='False', confirm=False,
+        quiet=False):
+
+    if config_file is not None:
+        with open(config_file, "r") as f:
+            config = yaml.safe_load(f)
+        if not dataset_names:
+            dataset_names = config.get("dataset", tuple())
+        if not solver_names:
+            solver_names = config.get("solver", tuple())
+            forced_solvers = config.get("force-solver", tuple())
+            solver_names = list(set(solver_names).union(set(forced_solvers)))
 
     # Check that the dataset/solver patterns match actual dataset
     benchmark = Benchmark(benchmark)
-    print(f"Installing '{benchmark.name}' requirements")
     benchmark.validate_dataset_patterns(dataset_names)
     benchmark.validate_solver_patterns(solver_names)
 
     # Get a list of all conda envs
     default_conda_env, conda_envs = list_conda_envs()
 
-    # If env_name is False (default), installation in the current environement.
+    # check if any current conda environment
+    if default_conda_env is None:
+        raise RuntimeError(
+            "No conda environment is activated. "
+            "You should be in a conda environment to use "
+            "'benchopt install'."
+        )
+
+    print(f"Installing '{benchmark.name}' requirements")
+    # If env_name is False (default), installation in the current environment.
     if env_name == 'False':
         env_name = None
         # incompatible with the 'recreate' flag to avoid messing with the
-        # user environement
+        # user environment
         if recreate:
             msg = "Cannot recreate conda env without using options " + \
                 "'-e/--env' or '--env-name'."
             raise RuntimeError(msg)
 
-        # check if any current conda environment
-        if default_conda_env is not None:
-            # ask for user confirmation to install in current conda env
-            if not confirm:
-                click.confirm(
-                    f"Install in the current env '{default_conda_env}'?",
-                    abort=True
-                )
-        else:
-            raise RuntimeError("No conda environment is activated.")
+        # ask for user confirmation to install in current conda env
+        if not confirm:
+            click.confirm(
+                f"Install in the current env '{default_conda_env}'?",
+                abort=True
+            )
+
     else:
         # If env_name is True, the flag `--env` has been used. Create a conda
         # env specific to the benchmark. Else, use the <env_name> value.
@@ -379,7 +475,7 @@ def install(benchmark, minimal, solver_names, dataset_names, force=False,
     "benchopt.github.io/how.html#example-of-parametrized-simulated-dataset",
     context_settings=dict(ignore_unknown_options=True)
 )
-@click.argument('benchmark', type=click.Path(exists=True),
+@click.argument('benchmark', default=Path.cwd(), type=click.Path(exists=True),
                 shell_complete=complete_benchmarks)
 @click.option('--env-name', type=str, default=None, metavar='NAME',
               shell_complete=complete_conda_envs,
